﻿#region Using

using System;
using System.Web;
using System.Web.UI;
using System.IO;
using System.Collections.Generic;
using BlogEngine.Core;
using wpCumulus;

#endregion

namespace Controls
{
    public class FlashTagCloud : WPCumulus
    {

        static FlashTagCloud()
        {
            Post.Saved += delegate { _tagList = null; };
        }

        #region Private fields

        private static List<TagInfo> _tagList;
        private static object _SyncRoot = new object();

        #endregion


        private List<TagInfo> TagList
        {
          get
          {
                if (_tagList == null)
                {
                    lock (_SyncRoot)
                    {
                        if (_tagList == null)
                        {
                            List<TagInfo> tagList = new List<TagInfo>();

                            TagInfo tagInfo;
                       
                            SortedDictionary<string, int> dic = CreateRawList();

                            foreach (string tag in dic.Keys)
                            {
                                tagInfo = new TagInfo();
                                tagInfo.Link = Utils.AbsoluteWebRoot + "?tag=/" + HttpUtility.UrlEncode(tag);
                                tagInfo.Tag = tag;
                                tagInfo.Weight = dic[tag];
                                tagList.Add(tagInfo);
                            }

                            _tagList = tagList;
                        }
                    }
                }

                return _tagList;
          }
    }

    /// <summary>
    /// Builds a raw list of all tags and the number of times
    /// they have been added to a post.
    /// </summary>
    private static SortedDictionary<string, int> CreateRawList()
    {
        SortedDictionary<string, int> dic = new SortedDictionary<string, int>();
        foreach (Post post in Post.Posts)
        {
            if (post.IsVisibleToPublic)
            {
                foreach (string tag in post.Tags)
                {
                    if (dic.ContainsKey(tag))
                        dic[tag]++;
                    else
                        dic[tag] = 1;
                }
            }
        }
        return dic;
    }
    
    /// <summary>
    /// Renders the control.
    /// </summary>
    protected override void RenderContents(HtmlTextWriter writer)
    {

        writer.Write("<script type=\"text/javascript\" src=\"");
        writer.Write(Page.ClientScript.GetWebResourceUrl(typeof(WPCumulus), "wpCumulus.swfobject.js"));
        writer.Write("\"></script>");

        DataSource = TagList;
        List<object> items = new List<object>();
        foreach (object o in _tagList)
        {
            items.Add(o);
        }

        DataBind();
        base.RenderContents(writer);

    }

        public class TagInfo
        {
            private string _link;
            private string _tag;
            private int _weight;

            public String Link
            {
                get
                {
                    return _link;
                }
                set
                {
                    _link = value;
                }
            }

            public String Tag
            {
                get
                {
                    return _tag;
                }
                set
                {
                    _tag = value;
                }
            }

            public int Weight
            {
                get
                {
                    return _weight;
                }
                set
                {
                    _weight = value;
                }
            }
        }

  }
}